import validateTasks from './routes/validate-tasks.js';
import language from './helper/language.js';
import onBefore from './messaging/before.js';
import onAfter from './messaging/after.js';
import onWorker from './messaging/worker.js';
import onCalculate from './messaging/calculator.js';
import Counter from './helper/counter.js';
import buildWorker from './worker/worker-factory.js';
import { EMPTY, } from './constants.js';
import buildTaskList from './routes/build-task-list.js';
/* eslint max-params:0 */
const executor = (threads, repetitions, job, resultHandler, logger, Worker, reportModifiers, resultStorage, resultOutputDir, progress, blacklist) => {
    const total = threads * repetitions;
    const now = new Date();
    validateTasks(repetitions, threads, job.main);
    const results = {};
    const finished = {};
    logger.debug(language('initialization', `${repetitions}`, `${threads}`));
    const internalTasks = buildTaskList(job.main, blacklist, total);
    progress.start(job, repetitions, threads);
    const calculator = buildWorker(Worker, 'calculator', (data, self) => onCalculate(data, finished, now, resultStorage, logger, progress, self, reportModifiers, resultHandler, resultOutputDir));
    const after = buildWorker(Worker, 'webrequest', (data, self) => onAfter(progress, job, logger, self));
    const startMain = () => {
        logger.debug(language('starting_workers', `${threads}`));
        for (let j = 0; j < threads; j++) {
            const worker = buildWorker(Worker, 'webrequest', (data, self) => onWorker(data, progress, logger, results, total, calculator, internalTasks, self, after, job));
            Counter.increment('active');
            worker.postMessage(internalTasks.shift());
        }
    };
    const before = buildWorker(Worker, 'webrequest', (data, self) => onBefore(progress, job, logger, self, startMain));
    if (job.before.length > EMPTY) {
        logger.debug(language('next_request'));
        before.postMessage(job.before.shift());
    }
    else {
        before.terminate();
        startMain();
    }
};
export default executor;
