import language from '../helper/language.js';
import ResultSet from './result-set.js';
import store from '../store/store.js';
import Counter from '../helper/counter.js';
import { EMPTY, } from '../constants.js';
const startAfter = (after, logger, progress, job) => {
    if (!Counter.isEmpty('active')) {
        return;
    }
    if (job.after.length > EMPTY) {
        logger.debug(language('starting_after'));
        after.postMessage(job.after.shift());
        return;
    }
    logger.debug(language('no_after'));
    after.terminate();
    store.clean();
    progress.stop();
};
const startAnalyzing = (id, total, logger, calculator, results) => {
    if (results[id].count !== total) {
        return;
    }
    logger.info(language('finished_requests', id));
    logger.debug(language('starting_analyzation', id));
    Counter.increment('analyzing');
    calculator.postMessage(results[id]);
};
const onWorker = (data, progress, logger, results, total, calculator, internalTasks, worker, after, job) => {
    logger.debug(language('starting_validation', data.id));
    results[data.id] = results[data.id] || new ResultSet(data.id);
    results[data.id].add(data);
    progress.increment();
    startAnalyzing(data.id, total, logger, calculator, results);
    if (internalTasks.length > EMPTY) {
        logger.debug(language('next_request'));
        worker.postMessage(internalTasks.shift());
        return;
    }
    Counter.decrement('active');
    logger.info(language('end_thread'));
    startAfter(after, logger, progress, job);
    worker.terminate();
};
export default onWorker;
