import Result from '../messaging/result.js';
import { request, } from 'needle';
import load from '../routes/middleware-loader.js';
const buildAnswer = (result, msg, success) => ({
    duration: result.duration,
    id: result.id,
    success,
    msg,
});
const handlePre = async (task) => {
    if (task.pre) {
        for (const middleware of task.pre) {
            // eslint-disable-next-line no-await-in-loop
            const requestMiddleware = await load(middleware, 'pre');
            task.main = requestMiddleware(task.main);
        }
    }
    return task.main;
};
const handlePost = async (task, res, callable) => {
    if (task.post) {
        for (const validator of task.post) {
            try {
                // eslint-disable-next-line no-await-in-loop
                const validatorMiddleware = await load(validator, 'post');
                validatorMiddleware(res);
            }
            catch (er) {
                callable(buildAnswer(res, er + '', false));
                return false;
            }
        }
    }
    return true;
};
export default async (task, callable) => {
    const requestConfig = await handlePre(task);
    const startTime = process.hrtime();
    request(requestConfig.method, requestConfig.url, requestConfig.body, {
        headers: requestConfig.headers,
        cookies: requestConfig.cookies,
    }, async (error, result) => {
        if (error) {
            callable(buildAnswer({
                duration: null,
                id: task.id,
                success: false,
                msg: error,
                validators: [],
                // eslint-disable-next-line no-undefined
                maxDuration: undefined,
            }, error + '', false));
            return;
        }
        const httpResult = new Result(task.id, task.main.url, startTime, process.hrtime(), result, task.post || [], task.main.maxDuration);
        if (await handlePost(task, httpResult, callable)) {
            callable(buildAnswer(httpResult, '', true));
        }
    });
};
